#!/bin/sh
# this file is sourced not run

# Set traps and create temporary folder. {{{1
TMPD=$(mktemp -d -p "${TMPDIR:-/tmp}" "${0##*/}_XXXXXX")
# created with permissions u+rwx minus umask restrictions
chmod 700 "$TMPD" || exit 1
trap "[ \${STATUS:?} -eq 0 ] && rm -rf \"${TMPD:?}\"; exit \$STATUS" \
	HUP INT ALRM TERM 0 # USR1 USR2

# {{{1}}}
mkdir -p "$TMPD"

MDVIEW=${MDVIEW:-./mdview}

TEST_EXPECT_DIR='test/expect'
TEST_OUTS="$TMPD/${TEST_LABEL:?}"
TEST_EXPECTS="${TEST_EXPECT_DIR:?}/${TEST_LABEL:?}"

# $* html|tty|text ...
test_create_expect () { # => STATUS
	local p ret
	STATUS=0
	for p in "${@:-html}"; do
		ret=1
		${MDVIEW:?} $TEST_ARGS --$p > "${TEST_EXPECTS:?}.$p"
		ret=$?
		[ $ret -eq 0 ] &&
			printf "\033[7m  %s  \033[0m %s %s\n" \
			"CREATED" "${TEST_LABEL:?}" "$p" ||
			printf "\033[7;31m  %s  \033[0m %s %s\n" \
			"ERROR" "${TEST_LABEL:?}" "$p"
		[ $ret -eq 0 ] || STATUS=1
	done
	return $STATUS
}

# $* html|tty|text ...
run_test () {
	local p ret
	for p in "${@:-html}"; do
		${MDVIEW:?} $TEST_ARGS --$p > "${TEST_OUTS:?}.$p"
		[ $? -eq 0 ] || ret=1
	done
	return $ret
}

# $* html|tty|text ...
run_test_diffs () {
	local ret
	for p in "${@:-html}"; do
			test_diff "${TEST_EXPECTS:?}.$p" "${TEST_OUTS:?}.$p"
		[ $? -eq 0 ] || ret=1
	done
	return $ret
}

# $1-expected_output_file(original) $2-test_output_file(temp)
test_diff () {
	test_diff_default "${TEST_EXPECTS:?}.$p" "${TEST_OUTS:?}.$p"
}

# $1-expected_output_file(original) $2-test_output_file(temp)
test_diff_default () {
	diff "${TEST_EXPECTS:?}.$p" "${TEST_OUTS:?}.$p"
}

# $1-expected_output_file(original) $2-test_output_file(temp)
test_diffy_default () {
	diff -y --suppress-common-lines "$1" "$2"
}

# $1-expected_output_file(original) $2-test_output_file(temp)
test_diffc_default () {
	git -P diff --word-diff-regex="[A-Za-z0-9. ]|[^[:space:]]" -- "$1" "$2"
		# If you do not like the extra [- or {+ add option --color-words
}

# $* html|tty|text ...
main () { # => $STATUS
	local p failed
	STATUS=0
	for p in "${@:-html}"; do
		run_test "$p"
		if [ $? -ne 0 ]; then
			printf "\033[7m  %s  \033[0m %s %s\n" \
			"INVALID" "${TEST_LABEL:?}" "$p"
			false
		else
			run_test_diffs "$p"
			failed=$?
			if [ $failed -eq 0 ]; then
				printf "\033[7m  %s  \033[0m %s %s\n" \
				"PASS" "${TEST_LABEL:?}" "$p"
			else
				STATUS=1
				printf "\033[7;31m  %s  \033[0m %s %s\n" \
				"FAIL $failed" "${TEST_LABEL:?}" "$p"
			fi
		fi
	done
	return $STATUS
}
